@@ -30,5 +30,5 @@
 
 '''
 
-__version__ = (1, 1)
-__author__ = "Sam Gammon <sam@keen.io>"
+__version__ = (1, 2)
+__author__ = "Sam Gammon <sam@keen.io>, Cory G Watson <gphat@keen.io>"
