@@ -103,7 +103,8 @@ class Providers:
 
     ''' Enumerates available event providers. '''
 
-    pass
+    KEEN = 'keen'
+    MIXPANEL = 'mixpanel'
 
 
 class Provider(object):
@@ -677,6 +678,10 @@ class KeenUploader(Uploader):
             self.provider.lib.add_events(self.events_by_kind)  # add the events to Keen (this results in a web request)
         )
 
+        # Clear out the events-by-kind map so we can start
+        # over after committing.
+        self.events_by_kind = {}
+
         # return count of events we sent
         return count
 
@@ -764,7 +769,7 @@ class Mixpanel(Provider):
         self.logging.debug('Switched Mixpanel endpoint to "%s".' % self.client.ENDPOINT)
 
         if traceback: raise  # always re-raise exceptions (we can't stop, this is bat country)
-        
+
         return self
 
     # `__enter__`/`__exit__`: co-opt Python's `with` for endpoint switching
@@ -871,57 +876,52 @@ class MixpanelDownloader(Downloader):
 
         with self.provider as mixpanel:  # enter `export` mode, which switches endpoints
 
-            # export events for each kind in the date range
+            # XXX Mixpanel doesn't stream to disk, so this won't work for big stuff.
             result = mixpanel.client.request(['export'], {
                 'from_date': _TO_MIXPANEL_DATE(begin),
                 'to_date': _TO_MIXPANEL_DATE(end),
                 'event': kinds
             })
 
-            # check for mixpanel API errors
-            if 'error' in result:  # pragma: nocover
+            event_count, error_count, ignore_errors = 0, 0, False
 
-                # @TODO(sgammon): proper mixpanel error handling
-                if self.provider.bus.debug:
-                    import pdb; pdb.set_trace()
+            # with closing(result) as page:
+            for event in result:
+                if event == "":
+                    # Skip empty lines
+                    continue
 
-                raise RuntimeError(' '.join(('Mixpanel:', result['error'])))
+                event_count += 1
+                self.logging.debug('Got raw event: "%s".' % event)
 
-            else:  # no error ocurred: parse newline-separated JSON events
-                event_count, error_count, ignore_errors = 0, 0, False
-                for event in filter(lambda x: x, result.split("\n")):  # (subtly filter out falsy empty strings)
-                    
-                    event_count += 1
-                    self.logging.debug('Got raw event: "%s".' % event)
+                try:
 
-                    try:
+                    # try to deserialize
+                    deserialized = json.loads(event)
 
-                        # try to deserialize
-                        deserialized = json.loads(event)
+                except Exception as e:  # pragma: nocover
 
-                    except Exception as e:  # pragma: nocover
+                    self.logging.debug('Encountered exception: "%s".' % str(e))
+                    self.logging.error('Failed to decode JSON from blob.')
 
-                        self.logging.debug('Encountered JSON exception: "%s".' % str(e))
-                        self.logging.error('Failed to decode JSON from blob.')
+                    # prompt for what to do from here
+                    if (not ignore_errors) and self.provider.bus.prompt("\n\n`Importer` encountered an issue decoding the following blob:\n%s\n" % event, "Ignore errors?"):
+                        ignore_errors = True
 
-                        # prompt for what to do from here
-                        if (not ignore_errors) and self.provider.bus.prompt("\n\n`Importer` encountered an issue decoding the following blob:\n%s\n" % event, "Ignore errors?"):
-                            ignore_errors = True
+                    # fail if we've tried more than 5 times and every time has failed
+                    error_count += 1
+                    if (event_count > 5) and error_count == event_count:
+                        raise RuntimeError(' '.join(('Mixpanel:', 'too many errors.')))
 
-                        # fail if we've tried more than 5 times and every time has failed
-                        error_count += 1
-                        if (event_count > 5) and error_count == event_count:
-                            raise RuntimeError(' '.join(('Mixpanel:', 'too many errors.')))
+                else:
 
-                    else:
+                    # convert to python datetime
+                    ts = deserialized['properties']['time']
+                    if isinstance(ts, int):
+                        ts = datetime.fromtimestamp(ts)
 
-                        # convert to python datetime
-                        ts = deserialized['properties']['time']
-                        if isinstance(ts, int):
-                            ts = datetime.fromtimestamp(ts)
-
-                        # yield event to caller
-                        yield (deserialized['event'], deserialized['properties'], self.provider.timezone.localize(ts).astimezone(self.utc))
+                    # yield event to caller
+                    yield (deserialized['event'], deserialized['properties'], self.provider.timezone.localize(ts).astimezone(self.utc))
 
 
 # +=+=+=+ Importer +=+=+=+ #
@@ -1042,7 +1042,7 @@ class Importer(object):
             if self.argset and hasattr(self.argset, 'config_file') and self.argset.config_file:
                 config_path = self.argset.config_file
             else:
-                config_path = os.path.join(os.path.dirname(__file__), 'config.json')
+                config_path = os.path.join(os.getcwd(), 'config.json')
 
             # try loading config
             try:
@@ -1243,7 +1243,7 @@ class Importer(object):
 
         # grab kinds, to/from providers
         kinds, source, target, reset_ts = [arguments[x] for x in ('kinds', 'from', 'to', 'reset_ts')]
-        
+
         # resolve provider object
         source, target = self.provider(source), self.provider(target)
         self.logging.debug('Providers: SOURCE(%s) -> TARGET(%s)' % (source, target))
@@ -1272,29 +1272,45 @@ class Importer(object):
 
         # calculate buffer file path
         download_buffer_path = os.path.join(os.path.dirname(__file__), _CSV_FILE)
-        self.logging.debug('Buffering downloaded data to "%s"...' % download_buffer_path)
 
-        # open CSV and set up download buffer file
+        # We need a count of events, so init at 0
         event_count = 0
-        with open(download_buffer_path, 'wb') as download_buffer:
+        # We might need to skip some. Look for that.
+        event_skip = arguments.get('skip', 0)
 
-            # get CSV writer
-            writer = csv.writer(download_buffer, **_CSV_PARAMS)
+        # Check to see if the download file from a previous run is still around.
+        if(os.path.isfile(download_buffer_path)):
+            self.logging.info('Download file "%s" already exists, reading from that.' % download_buffer_path)
+            # Count the number of lines in the file.
+            with open(download_buffer_path) as f:
+                for i, l in enumerate(f):
+                    pass
+            event_count = i + 1
 
-            # fetch source events
-            self.say('Beginning event data download...')
-            for kind, data, timestamp in source.downloader.download(begin, end, kinds):
+        else:
+            # Nothing to continue from. Start over!
+            self.logging.debug('Buffering downloaded data to "%s"...' % download_buffer_path)
 
-                event_count += 1
+            # open CSV and set up download buffer file
+            with open(download_buffer_path, 'wb') as download_buffer:
 
-                # log about it
-                self.logging.info('Downloaded "%s" (event #%s @ %s bytes)...' % (kind, event_count, sys.getsizeof(data)))
-                self.logging.debug('Got event data: "%s".' % (str(data)))
+                # get CSV writer
+                writer = csv.writer(download_buffer, **_CSV_PARAMS)
 
-                # write to our buffer file
-                writer.writerow((kind, _TO_INTERNAL_DT(timestamp), json.dumps(source.downloader.post_process(data))))
+                # fetch source events
+                self.say('Beginning event data download...')
+                for kind, data, timestamp in source.downloader.download(begin, end, kinds):
 
-            self.logging.info('Download complete.')
+                    event_count += 1
+
+                    # log about it
+                    self.logging.info('Downloaded "%s" (event #%s @ %s bytes)...' % (kind, event_count, sys.getsizeof(data)))
+                    self.logging.debug('Got event data: "%s".' % (str(data)))
+
+                    # write to our buffer file
+                    writer.writerow((kind, _TO_INTERNAL_DT(timestamp), json.dumps(source.downloader.post_process(data))))
+
+                self.logging.info('Download complete.')
 
         if self.cli:
             time.sleep(2)  # give the user a chance to catch up
@@ -1319,6 +1335,12 @@ class Importer(object):
 
                 event_count += 1
 
+                if(event_skip > 0 and event_count <= event_skip):
+                    # We've not yet exceeded the number we were told to skip. So skip
+                    # this iteration
+                    self.logging.info('Skipping #%s, < %s' % (event_count, event_skip))
+                    continue
+
                 # extract info
                 kind, timestamp, data = event
 
@@ -1327,7 +1349,17 @@ class Importer(object):
 
                 target.uploader.upload(kind, target.uploader.pre_process(data), _FROM_INTERNAL_DT(timestamp) if not reset_ts else None)  # extract row data and upload
 
-            # dispatch commit(), if available
+                # If the count of events matches the batch size, commit!
+                if not event_count % 1000: # This is hardcoded for now to make things easier, XXX - @gphat
+                    try:
+                        self.say('Committing events...')
+                        target.uploader.commit()
+                        self.logging.info('Committed up to event %s' % event_count)
+                    except NotImplementedError:  # pragma: nocover
+                        self.logging.critical('Failed to commit events.')
+
+            # Now that we're outside of the loop, commit again to catch anything
+            # else we have lying around.
             try:
                 self.say('Committing events...')
                 target.uploader.commit()
@@ -1344,7 +1376,8 @@ class Importer(object):
                            "from %s to %s. The transferred data is currently\n "
                            "buffered in a file called '%s' - would you like to\n "
                            "keep it as a backup?" % (event_count, source.name.capitalize(), target.name.capitalize(), _CSV_FILE), "Keep file?"):
-
+            self.logging.info('Kept "%s" as a backup.' % _CSV_FILE)
+        else:
             try:
                 os.remove(download_buffer_path)  # delete the buffer file if requested
             except:  # pragma: nocover
@@ -1380,7 +1413,8 @@ class Importer(object):
             ``0`` is returned to indicate no error,
             otherwise ``1`` is returned. '''
 
-        self.cli, self.argset, self.debug = True, cli, self.config.get('debug', False)  # setup CLI and debug mode
+        self.cli, self.argset = True, cli
+        self.debug = self.config.get('debug', False)  # setup CLI and debug mode
 
         try:
 
@@ -1413,8 +1447,8 @@ class Importer(object):
                 'verbose': bool(cli.get('verbose', self.config.get('verbose', False))),  # verbose mode
                 'reset_ts': bool(cli.get('reset_ts', self.config.get('reset_timestamps', False))),  # whether to reset timestamps
                 'quiet': bool(cli.get('quiet', self.config.get('quiet', False))),  # quiet mode
-                'yes': bool(cli.get('yes', self.config.get('yes', False)))  # yes mode
-
+                'yes': bool(cli.get('yes', self.config.get('yes', False))),  # yes mode
+                'skip': cli.get('skip', 0) # skip some
             })) else 1
 
         except NotImplementedError as e:  # pragma: nocover
